﻿/*
 * TokyoTyrant.NET (http://tokyotyrant.codeplex.com)
 * Pure .NET interface to Tokyo Tyrant
 * Copyright (C) 2009 Russell van der Walt (http://blog.ruski.co.za)

 * TokyoTyrant.NET is free software; you can redistribute it and/or modify it under the terms of
 * the Microsoft Public License (Ms-PL). TokyoTyrant.NET is distributed in the hope
 * that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the Microsoft Public
 * License for more details.
 * 
 * Tokyo Cabinet and Tokyo Tyrant is Copyright (C) 2006-2008 Mikio Hirabayashi 
 * http://tokyocabinet.sourceforge.net/
 * 
 */

using System;
using System.Collections.Generic;
using System.Text;

namespace TokyoTyrant.NET {

    public class TokyoTyrantException : Exception { 
        
        public TokyoTyrantException(string message) : base(message) {
        }

        public static TokyoTyrantException FromErrorCode(int errorCode) {
            switch (errorCode) {
                case 1: // EINVALID = 1 # error code: invalid operation
                    return new TTInvalidOperationException(); 
                case 2:  // ENOHOST = 2 # error code: host not found
                    return new TTNoHostException();
                case 3: // EREFUSED = 3 # error code: connection refused
                    return new TTConnectionRefusedException();
                case 4:  // ESEND = 4 # error code: send error
                    return new TTSendException();
                case 5:  // ERECV = 5 # error code: recv error
                    return new TTReceiveException();
                case 6:  // EKEEP = 6 # error code: existing record
                    return new TTExistingRecordException();
                case 7:  // ENOREC = 7 # error code: no record found
                    return new TTRecordNotFoundException();
                case 9999: // EMISC = 9999 # error code: miscellaneous error
                    return new TTMiscException();
                default:
                    return new TokyoTyrantException("Error " + errorCode.ToString());
            }
        }
        
    
    }
    
    public class TTNotConnectedException : TokyoTyrantException { public TTNotConnectedException() : base("Not Connected") { } }
    public class TTInvalidOperationException : TokyoTyrantException { public TTInvalidOperationException() : base("Invalid Operation") { } }
    public class TTNoHostException : TokyoTyrantException { public TTNoHostException() : base("Host not found") { } }
    public class TTConnectionRefusedException : TokyoTyrantException { public TTConnectionRefusedException() : base("Connection refused") { } }    
    public class TTSendException : TokyoTyrantException { public TTSendException() : base("Send error") { } }
    public class TTReceiveException : TokyoTyrantException { public TTReceiveException() : base("Receive error") { } }
    public class TTExistingRecordException : TokyoTyrantException { public TTExistingRecordException() : base("Existing record") { } }
    public class TTRecordNotFoundException : TokyoTyrantException { public TTRecordNotFoundException() : base("Record not found") { } }
    public class TTMiscException : TokyoTyrantException { public TTMiscException() : base("Miscellaneous error") { } }
    




}
